from django.conf.urls import url

from slxauth.views import LoginView, CallbackView, LogoutView

""" Include in urlpatterns like this:
url(r'^accounts/', include('slxauth.urls', namespace='slxauth'))
"""

app_name = 'slxauth'

urlpatterns = [
    url(r'^login/$', LoginView.as_view(), name='login'),
    url(r'^logout/$', LogoutView.as_view(), name='logout'),
    url(r'^callback/$', CallbackView.as_view(), name='callback'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-22 12:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0006_auto_20171110_1501'),
    ]

    operations = [
        migrations.AddField(
            model_name='myuser',
            name='access_token',
            field=models.TextField(blank=True, null=True, verbose_name='OAuth2 Access Token'),
        ),
    ]



import logging

import requests
from django.conf import settings
from django.contrib.auth.views import LogoutView
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.urls import reverse
from django.views import View
from requests.auth import HTTPBasicAuth

from slxauth.utils import login_using_token

logger = logging.getLogger(__name__)


class LoginView(View):
    """ This is the view that django opens """
    def get(self, request):
        if not 'next' in request.GET:
            logger.debug('no next provided. this seems to be an unusual request.')
            return HttpResponseForbidden()

        next = request.GET['next']
        redirect_uri = request.build_absolute_uri('%s?next=%s' % (reverse('slxauth:callback'), next))
        authorize_url = '%s/authorize?client_id=%s&client_secret=%s&response_type=code&redirect_uri=%s' % (
            settings.OAUTH_URL,
            settings.OAUTH_CLIENT_ID,
            settings.OAUTH_CLIENT_SECRET,
            redirect_uri
        )
        logger.debug('redirecting to %s' % authorize_url)
        return HttpResponseRedirect(redirect_to=authorize_url)


class CallbackView(View):
    """ This handles the callback from the OAuth2 Auth server.
    Users will be redirected to this view after successful login.
    """
    def get(self, request):
        if not 'code' in request.GET:
            logger.debug('no code provided')
            return HttpResponseForbidden()

        if not 'next' in request.GET:
            logger.debug('no next provided. this seems to be an unusual request.')
            return HttpResponseForbidden()

        code = request.GET['code']
        next = request.GET['next']

        logger.debug('received callback code %s' % code)
        redirect_uri = request.build_absolute_uri('%s?next=%s' % (reverse('slxauth:callback'), next))
        url = '%s/token?grant_type=authorization_code&code=%s&redirect_uri=%s' % (settings.OAUTH_URL, code, redirect_uri)
        
        logger.debug('validating code using %s' % url)

        authresponse = None

        try:
            authresponse = requests.post(
                url,
                auth=HTTPBasicAuth(settings.OAUTH_CLIENT_ID, settings.OAUTH_CLIENT_SECRET),
                headers={'Content-Type': 'application/x-www-form-urlencoded'}, timeout=0.5)

            logger.debug('response: %s' % authresponse)
            logger.debug('response content: %s' % authresponse.content)
        except Exception as e:
            logger.error('could not validate token: %s' % e)

        if authresponse and authresponse.status_code == 200:
            data = authresponse.json()

            if 'access_token' in data and 'expires_in' in data and 'refresh_token' in data:
                access_token = data['access_token']
                expiration_time = data['expires_in']
                refresh_token = data['refresh_token']

                if login_using_token(request, access_token):
                    response = HttpResponseRedirect(next)
                    response.set_cookie('access_token', access_token, domain=settings.OAUTH_COOKIE_DOMAIN)
                    response.set_cookie('expiration_time', expiration_time, domain=settings.OAUTH_COOKIE_DOMAIN)
                    response.set_cookie('refresh_token', refresh_token, domain=settings.OAUTH_COOKIE_DOMAIN)
                    return response
        else:
            logger.debug('received response %s from auth service. can\'t login' % authresponse)

        return HttpResponseForbidden()


class LogoutView(LogoutView):
    def dispatch(self, request, *args, **kwargs):
        response = super(LogoutView, self).dispatch(request, *args, **kwargs)

        # clear the SSO cookies to logout from all portal apps
        response.set_cookie('access_token', max_age=0, domain=settings.OAUTH_COOKIE_DOMAIN)
        response.set_cookie('expiration_time', max_age=0, domain=settings.OAUTH_COOKIE_DOMAIN)
        response.set_cookie('refresh_token', max_age=0, domain=settings.OAUTH_COOKIE_DOMAIN)
        return response




import logging

import jwt
import requests
from django.conf import settings
from django.contrib.auth import get_user_model, login
from django.contrib.auth.models import Group


logger = logging.getLogger(__name__)


def getpk(minipk):
    """
    ### mySolarlux convention ###
    since the publickey is stored in a no-newline env-var we need to add a prefix and suffix to it,
    to be able to use it as a jwt publickey signature
    """
    return """-----BEGIN PUBLIC KEY-----
%s
-----END PUBLIC KEY-----""" % minipk


def get_unsaved_user_from_token(access_token, token_data):

    user_model = get_user_model()
    user, created = user_model.objects.get_or_create(email=token_data['email'])

    logger.debug('%s user %s' % ('creating' if created else 'updating', user.email))

    user.access_token = access_token
    user.customer_no = token_data['customerNumber']
    user.customer_name = token_data['customerName']
    user.last_name = token_data['lastName']
    user.first_name = token_data['firstName']
    user.title = token_data['title']
    user.is_customer_admin = token_data['isCustomerAdmin']
    user.language = token_data['language']
    user.crm_contact_id = token_data['crmContactId']
    user.um_id = token_data['id']

    logger.debug('creating groups for user %s' % user.email)
    for role in token_data['authorities']:
        group, created = Group.objects.get_or_create(name=role)
        user.groups.add(group)
    
    user.is_active = len(settings.OAUTH_ROLES_REQUIRED) == 0 or len(set(settings.OAUTH_ROLES_REQUIRED) & set(token_data['authorities'])) > 0
    user.is_staff = len(set(settings.OAUTH_ROLES_STAFF) & set(token_data['authorities'])) > 0
    user.is_superuser = len(set(settings.OAUTH_ROLES_SUPERUSER) & set(token_data['authorities'])) > 0

    logger.debug('finished user extraction from token')
    return user

def get_unsaved_user_from_token_and_portal_config(access_token, token_data, portal_config):
    user = get_unsaved_user_from_token(access_token, token_data)

    if portal_config and 'fetch-user-info-url' in portal_config:
        logger.debug('fetching user info from portal config for %s' % user.email)
        user_profile_response = None

        try:
            user_profile_response = requests.get(portal_config['fetch-user-info-url'], headers={'Authorization': 'Bearer %s' % access_token}, timeout=0.5)
            logger.debug('user profile response: %s' % user_profile_response)
        except Exception as e:
            logger.error('could not fetch user info: %s' % e)

        if user_profile_response and user_profile_response.status_code == 200:
            user_profile = user_profile_response.json()
            user.avatar_attachment_id = user_profile['avatarAttachmentId']
            user.company_logo_attachment_id = user_profile['companyLogoAttachmentId']
            logger.debug('updated user %s from portal config' % user.email)

    return user

def login_using_token(request, access_token):

    logger.debug('extracting user info from access-token %s' % access_token)
    oauth_public_key = getpk(settings.OAUTH_PUBLIC_KEY)
    try:
        # validate the oauth2 token
        token_data = jwt.decode(access_token, oauth_public_key, algorithms=['RS256'])
    except:
        return False

    portal_config_response = None

    logger.debug('beginning user login from token')
    try:
        # load more infos...
        portal_config_response = requests.get(settings.PORTAL_CONFIG_URL, headers={
            'Authorization': 'Bearer %s' % access_token
        }, timeout=0.5)
        logger.debug('portal config response: %s' % portal_config_response)
    except Exception as e:
        logger.error('could not load remote portal config: %s' % e)

    user = None

    if portal_config_response and portal_config_response.status_code == 200:
        try:
            logger.debug('getting user from access token and portal config')
            portal_config = portal_config_response.json()
            user = get_unsaved_user_from_token_and_portal_config(access_token, token_data, portal_config)
        except Exception as e:
            logger.error('could not get user from access token and portal config: %s' % e)

    if not user:
        logger.debug('getting user from access token without portal config')
        user = get_unsaved_user_from_token(access_token, token_data)

    logger.debug('saving user object')
    user.save()
    logger.debug('logging in user %s' % user.email)
    login(request, user)
    return True

